import { Rule } from './Rule';
import { RawRuleFrom } from './RawRule';
import { DetectSubjectType, ResolveAction, CanParameters, Abilities, Normalize, ConditionsMatcher, FieldMatcher } from './types';
export interface AnyRuleIndex<A extends Abilities = any, C = any> {
    /** hack property type to improve inference */
    readonly _za: A;
    /** hack property type to improve inference */
    readonly _zc: C;
    update(rules: RawRuleFrom<A, C>[]): this;
    possibleRulesFor(...args: CanParameters<A, false>): Rule<A, C>[];
    rulesFor(...args: CanParameters<A>): Rule<A, C>[];
    on<T extends keyof EventsMap<this>>(event: T, handler: EventHandler<EventsMap<this>[T]>): Unsubscribe;
}
export interface RuleIndexOptions<A extends Abilities, Conditions> {
    /** @deprecated use "detectSubjectType" option instead */
    subjectName?: this['detectSubjectType'];
    detectSubjectType?: DetectSubjectType<Normalize<A>[1]>;
    conditionsMatcher?: ConditionsMatcher<Conditions>;
    fieldMatcher?: FieldMatcher;
    resolveAction?: ResolveAction<Normalize<A>[0]>;
}
export declare type Generics<T extends AnyRuleIndex> = T extends AnyRuleIndex ? {
    abilities: T['_za'];
    conditions: T['_zc'];
} : never;
export declare type RuleOf<T extends AnyRuleIndex> = Rule<Generics<T>['abilities'], Generics<T>['conditions']>;
export declare type RawRuleOf<T extends AnyRuleIndex> = RawRuleFrom<Generics<T>['abilities'], Generics<T>['conditions']>;
export declare type RuleIndexOptionsOf<T extends AnyRuleIndex> = RuleIndexOptions<Generics<T>['abilities'], Generics<T>['conditions']>;
interface Event<T> {
    ability: T;
}
export interface UpdateEvent<T extends AnyRuleIndex> extends Event<T> {
    rules: RawRuleOf<T>[];
}
export declare type EventHandler<Event> = (event: Event) => void;
declare type EventsMap<T extends AnyRuleIndex> = {
    update: UpdateEvent<T>;
    updated: UpdateEvent<T>;
};
export declare type Unsubscribe = () => void;
export declare class RuleIndex<A extends Abilities, Conditions> implements AnyRuleIndex {
    private _hasPerFieldRules;
    private _mergedRules;
    private _events;
    private _indexedRules;
    private readonly _ruleOptions;
    readonly detectSubjectType: DetectSubjectType<Normalize<A>[1]>;
    private _rules;
    readonly rules: RawRuleFrom<A, Conditions>[];
    /** hack property type to improve inference */
    readonly _za: A;
    /** hack property type to improve inference */
    readonly _zc: Conditions;
    constructor(rules?: RawRuleFrom<A, Conditions>[], options?: RuleIndexOptions<A, Conditions>);
    update(rules: RawRuleFrom<A, Conditions>[]): this;
    private _buildIndexFor;
    possibleRulesFor(...args: CanParameters<A, false>): Rule<A, Conditions>[];
    private _mergeRulesFor;
    rulesFor(...args: CanParameters<A>): Rule<A, Conditions>[];
    on<T extends keyof EventsMap<this>>(event: T, handler: EventHandler<EventsMap<this>[T]>): Unsubscribe;
    private _emit;
}
export {};
